@@ -8,9 +8,13 @@ Catmandu::Z3950 - Catmandu module for working with Z3950 data
 
 Version 0.04
 
+=head1 SYNOPSIS
+
+  $ catmandu convert Z3950 --host z3950.loc.gov --port 7090 --databaseName Voyager --query "(title = dinosaur)"
+
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 MODULES
 
@@ -24,10 +28,10 @@ our $VERSION = '0.04';
 
 =over
 
-=item * Patrick Hochstenbach, C<< <patrick.hochstenbach@ugent.be> >>
-
 =item * Wouter Willaert, C<< <wouterw@inuits.eu> >>
 
+=item * Patrick Hochstenbach, C<< <patrick.hochstenbach@ugent.be> >>
+
 =back
 
 =head1 LICENSE AND COPYRIGHT
